
Imports System.Collections
Imports System.Collections.Generic

Namespace Generic
    Partial Public Class Deque(Of T)

        ' ----------------------------------------------- Implements a synchronization wrapper around a deque
        <Serializable()> _
        Private Class SynchronizedDeque
            Inherits Deque(Of T)
            Implements IEnumerable

            ' ------------------------------------------- The wrapped deque.
            Private _deque As Deque(Of T)
            ' ------------------------------------------- The object to lock on.
            Private root As Object


            Public Sub New(ByVal deque As Deque(Of T))
                If deque Is Nothing Then
                    Throw New ArgumentNullException("deque")
                End If

                Me._deque = _deque
                Me.root = _deque.SyncRoot
            End Sub


            Public Overloads Overrides Sub Clear()
                SyncLock root
                    _deque.Clear()
                End SyncLock
            End Sub

            Public Overloads Overrides Function Contains(ByVal item As T) As Boolean
                SyncLock root
                    Return _deque.Contains(item)
                End SyncLock
            End Function

            Public Overloads Overrides Sub PushFront(ByVal item As T)
                SyncLock root
                    _deque.PushFront(item)
                End SyncLock
            End Sub

            Public Overloads Overrides Sub PushBack(ByVal item As T)
                SyncLock root
                    _deque.PushBack(item)
                End SyncLock
            End Sub

            Public Overloads Overrides Function PopFront() As T
                SyncLock root
                    Return _deque.PopFront()
                End SyncLock
            End Function

            Public Overloads Overrides Function PopBack() As T
                SyncLock root
                    Return _deque.PopBack()
                End SyncLock
            End Function

            Public Overloads Overrides Function PeekFront() As T
                SyncLock root
                    Return _deque.PeekFront()
                End SyncLock
            End Function

            Public Overloads Overrides Function PeekBack() As T
                SyncLock root
                    Return _deque.PeekBack()
                End SyncLock
            End Function

            Public Overloads Overrides Function ToArray() As T()
                SyncLock root
                    Return _deque.ToArray()
                End SyncLock
            End Function

            Public Overloads Overrides Function Clone() As Object
                SyncLock root
                    Return _deque.Clone()
                End SyncLock
            End Function

            Public Overloads Overrides Sub CopyTo(ByVal array As Array, ByVal index As Integer)
                SyncLock root
                    _deque.CopyTo(array, index)
                End SyncLock
            End Sub

            Public Overloads Overrides Function GetEnumerator() As IEnumerator(Of T)
                SyncLock root
                    Return _deque.GetEnumerator()
                End SyncLock
            End Function

            ' ------------------------------------- Returns an enumerator that can iterate through the Deque.
            'TODO ------------------------------------------------------ LIVIO - REMOVED
            'Private Overloads Function IEnumerable_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
            '    SyncLock root
            '        Return DirectCast(_deque, IEnumerable).GetEnumerator()
            '    End SyncLock
            'End Function

            Public Overloads Overrides ReadOnly Property Count() As Integer
                Get
                    SyncLock root
                        Return _deque.Count
                    End SyncLock
                End Get
            End Property

            Public Overloads Overrides ReadOnly Property IsSynchronized() As Boolean
                Get
                    Return True
                End Get
            End Property

        End Class

    End Class

End Namespace
