

Imports System.Collections
Imports System.Runtime.InteropServices

' ----------------------------------------------------------- builds a pointer to a MidiHeader structure
Friend Class MidiHeaderBuilder

    ' ------------------------------------------------------- the length of the system exclusive buffer
    Private m_bufferLength As Integer
    ' ------------------------------------------------------- the system exclusive data
    Private data As Byte()
    ' ------------------------------------------------------- the pointer to the MidiHeader has been built
    Private built As Boolean = False
    ' ------------------------------------------------------- the built pointer to the MidiHeader
    Private m_result As IntPtr


    Public Sub New()
        BufferLength = 1
    End Sub


    Public Sub Build()
        Dim header As New MidiHeader()
        ' -------------------------------------------------- initialize the MidiHeader
        header.bufferLength = BufferLength
        header.bytesRecorded = 0
        header.data = Marshal.AllocHGlobal(BufferLength)
        header.flags = 0
        ' -------------------------------------------------- write data to the MidiHeader
        For i As Integer = 0 To BufferLength - 1
            Marshal.WriteByte(header.data, i, data(i))
        Next
        Try
            m_result = Marshal.AllocHGlobal(Marshal.SizeOf(GetType(MidiHeader)))
        Catch generatedExceptionName As Exception
            Marshal.FreeHGlobal(header.data)
            Throw
        End Try
        Try
            Marshal.StructureToPtr(header, m_result, False)
        Catch generatedExceptionName As Exception
            Marshal.FreeHGlobal(header.data)
            Marshal.FreeHGlobal(m_result)
            Throw
        End Try
        built = True
    End Sub


    Public Sub InitializeBuffer(ByVal message As SysExMessage)
        ' ------------------------------------------------------ if this is a start system exclusive message
        If message.SysExType = SysExType.Start Then
            BufferLength = message.Length
            ' -------------------------------------------------- copy entire message
            For i As Integer = 0 To BufferLength - 1
                data(i) = message(i)
            Next
            ' -------------------------------------------------- else this is a continuation message
        Else
            BufferLength = message.Length - 1
            ' -------------------------------------------------- copy all but the first byte of message
            For i As Integer = 0 To BufferLength - 1
                data(i) = message(i + 1)
            Next
        End If
    End Sub

    Public Sub InitializeBuffer(ByVal events As ICollection)
        If events Is Nothing Then
            Throw New ArgumentNullException("events")
        ElseIf events.Count Mod 4 <> 0 Then
            Throw New ArgumentException("Stream events not word aligned.")
        End If
        If events.Count = 0 Then
            Return
        End If
        BufferLength = events.Count
        events.CopyTo(data, 0)
    End Sub

    Public Sub Destroy()
        If Not built Then
            Throw New InvalidOperationException("Cannot destroy MidiHeader")
        End If
        Destroy(m_result)
    End Sub

    Public Sub Destroy(ByVal headerPtr As IntPtr)
        Dim header As MidiHeader = CType(Marshal.PtrToStructure(headerPtr, GetType(MidiHeader)), MidiHeader)

        Marshal.FreeHGlobal(header.data)
        Marshal.FreeHGlobal(headerPtr)
    End Sub


    Public Property BufferLength() As Integer
        Get
            Return m_bufferLength
        End Get
        Set(ByVal value As Integer)
            If value <= 0 Then
                Throw New ArgumentOutOfRangeException("BufferLength", value, "MIDI header buffer length out of range.")
            End If
            m_bufferLength = value
            data = New Byte(value - 1) {}
        End Set
    End Property

    ' ------------------------------------------------------- gets the pointer to the MidiHeader
    Public ReadOnly Property Result() As IntPtr
        Get
            Return m_result
        End Get
    End Property

End Class
