

Imports LC.Multimedia

' --------------------------------------------------------------------- builds key signature MetaMessages
Public Class KeySignatureBuilder
    Implements IMessageBuilder

	Private m_key As Key = Key.CMajor

	Private m_result As MetaMessage = Nothing

	Public Sub New()
	End Sub

	Public Sub New(message As MetaMessage)
		Initialize(message)
	End Sub


    ' ---------------------------------------------- initializes the KeySignatureBuilder with the specified MetaMessage
	Public Sub Initialize(message As MetaMessage)
        If message Is Nothing Then
            Throw New ArgumentNullException("message")
        ElseIf message.MetaType <> MetaType.KeySignature Then
            Throw New ArgumentException("Wrong meta event type.", "messaege")
        End If
        '
        Dim b As SByte = CSByte(message(0))
        ' ---------------------------------------------------------- if the key is major
        If message(1) = 0 Then
            Select Case b
                Case -7 : m_key = Key.CFlatMajor
                Case -6 : m_key = Key.GFlatMajor
                Case -5 : m_key = Key.DFlatMajor
                Case -4 : m_key = Key.AFlatMajor
                Case -3 : m_key = Key.EFlatMajor
                Case -2 : m_key = Key.BFlatMajor
                Case -1 : m_key = Key.FMajor
                Case 0 : m_key = Key.CMajor
                Case 1 : m_key = Key.GMajor
                Case 2 : m_key = Key.DMajor
                Case 3 : m_key = Key.AMajor
                Case 4 : m_key = Key.EMajor
                Case 5 : m_key = Key.BMajor
                Case 6 : m_key = Key.FSharpMajor
                Case 7 : m_key = Key.CSharpMajor
            End Select
        Else
            ' ------------------------------------------------------- else the key is minor
            Select Case b
                Case -7 : m_key = Key.AFlatMinor
                Case -6 : m_key = Key.EFlatMinor
                Case -5 : m_key = Key.BFlatMinor
                Case -4 : m_key = Key.FMinor
                Case -3 : m_key = Key.CMinor
                Case -2 : m_key = Key.GMinor
                Case -1 : m_key = Key.DMinor
                Case 0 : m_key = Key.AMinor
                Case 1 : m_key = Key.EMinor
                Case 2 : m_key = Key.BMinor
                Case 3 : m_key = Key.FSharpMinor
                Case 4 : m_key = Key.CSharpMinor
                Case 5 : m_key = Key.GSharpMinor
                Case 6 : m_key = Key.DSharpMinor
                Case 7 : m_key = Key.ASharpMinor
            End Select
        End If
    End Sub


	Public Property Key() As Key
		Get
			Return m_key
		End Get
		Set
			m_key = value
		End Set
    End Property

    ' -------------------------------------------------------------- the build key signature MetaMessage
	Public ReadOnly Property Result() As MetaMessage
		Get
			Return m_result
		End Get
	End Property


    ' ------------------------------------------------------------------ Builds the key signature MetaMessage
    Public Sub Build() Implements IMessageBuilder.Build
        Dim data As Byte() = New Byte(MetaMessage.KeySigLength - 1) {}

        Select Case Key
            Case Key.CFlatMajor
                data(0) = 249 'CByte(-7)
                data(1) = 0

            Case Key.GFlatMajor
                data(0) = 250 'CByte(-6)
                data(1) = 0

            Case Key.DFlatMajor
                data(0) = 251 'CByte(-5)
                data(1) = 0

            Case Key.AFlatMajor
                data(0) = 252 'CByte(-4)
                data(1) = 0

            Case Key.EFlatMajor
                data(0) = 253 'CByte(-3)
                data(1) = 0

            Case Key.BFlatMajor
                data(0) = 254 'CByte(-2)
                data(1) = 0

            Case Key.FMajor
                data(0) = 255 'CByte(-1)
                data(1) = 0

            Case Key.CMajor
                data(0) = 0
                data(1) = 0

            Case Key.GMajor
                data(0) = 1
                data(1) = 0

            Case Key.DMajor
                data(0) = 2
                data(1) = 0

            Case Key.AMajor
                data(0) = 3
                data(1) = 0

            Case Key.EMajor
                data(0) = 4
                data(1) = 0

            Case Key.BMajor
                data(0) = 5
                data(1) = 0

            Case Key.FSharpMajor
                data(0) = 6
                data(1) = 0

            Case Key.CSharpMajor
                data(0) = 7
                data(1) = 0

            Case Key.AFlatMinor
                data(0) = 249 'CByte(-7)
                data(1) = 1

            Case Key.EFlatMinor
                data(0) = 250 'CByte(-6)
                data(1) = 1

            Case Key.BFlatMinor
                data(0) = 251 'CByte(-5)
                data(1) = 1

            Case Key.FMinor
                data(0) = 252 'CByte(-4)
                data(1) = 1

            Case Key.CMinor
                data(0) = 253 'CByte(-3)
                data(1) = 1

            Case Key.GMinor
                data(0) = 254 'CByte(-2)
                data(1) = 1

            Case Key.DMinor
                data(0) = 255 'CByte(-1)
                data(1) = 1

            Case Key.AMinor
                data(0) = 0
                data(1) = 1

            Case Key.EMinor
                data(0) = 1
                data(1) = 1

            Case Key.BMinor
                data(0) = 2
                data(1) = 1

            Case Key.FSharpMinor
                data(0) = 3
                data(1) = 1

            Case Key.CSharpMinor
                data(0) = 4
                data(1) = 1

            Case Key.GSharpMinor
                data(0) = 5
                data(1) = 1

            Case Key.DSharpMinor
                data(0) = 6
                data(1) = 1

            Case Key.ASharpMinor
                data(0) = 7
                data(1) = 1

        End Select

        m_result = New MetaMessage(MetaType.KeySignature, data)
    End Sub

End Class
