


' -------------------------------------------------- the basic class for all MIDI short messages.
'
'  MIDI short messages represent all MIDI messages except meta messages
'  and system exclusive messages. This includes channel messages, system
'  realtime messages, and system common messages.
'
Public MustInherit Class ShortMessage
    Implements IMidiMessage

    Public Const DataMaxValue As Integer = 127
    Public Const StatusMaxValue As Integer = 255

    Private Const StatusMask As Integer = Not 255
    Protected Const DataMask As Integer = Not StatusMask
    Private Const Data1Mask As Integer = Not 65280
    Private Const Data2Mask As Integer = 65535 'Not Data1Mask + DataMask ' <-- ( not working )
    Private Const Shift As Integer = 8

    Protected msg As Integer = 0

    ' -------------------------------------------------------------------------------------- "Methods"

    Public Function GetBytes() As Byte() Implements IMidiMessage.GetBytes
        Return BitConverter.GetBytes(msg)
    End Function

    Friend Shared Function PackStatus(ByVal message As Integer, ByVal status As Integer) As Integer
        If status < 0 OrElse status > StatusMaxValue Then
            Throw New ArgumentOutOfRangeException("status", status, "Status value out of range.")
        End If
        Return (message And StatusMask) Or status
    End Function

    Friend Shared Function PackData1(ByVal message As Integer, ByVal data1 As Integer) As Integer
        If data1 < 0 OrElse data1 > DataMaxValue Then
            Throw New ArgumentOutOfRangeException("data1", data1, "Data 1 value out of range.")
        End If
        Return (message And Data1Mask) Or (data1 << Shift)
    End Function

    Friend Shared Function PackData2(ByVal message As Integer, ByVal data2 As Integer) As Integer
        If data2 < 0 OrElse data2 > DataMaxValue Then
            Throw New ArgumentOutOfRangeException("data2", data2, "Data 2 value out of range.")
        End If
        Return (message And Data2Mask) Or (data2 << (Shift * 2))
    End Function

    Friend Shared Function UnpackStatus(ByVal message As Integer) As Integer
        Return message And DataMask
    End Function

    Friend Shared Function UnpackData1(ByVal message As Integer) As Integer
        Return (message And Not Data1Mask) >> Shift
    End Function

    Friend Shared Function UnpackData2(ByVal message As Integer) As Integer
        Return (message And Not Data2Mask) >> (Shift * 2)
    End Function


    ' ------------------------------------------------------------------------------------- "Properties"


    ' ---------------------------------------------------------- gets the short message as a packed integer
    Public ReadOnly Property Message() As Integer
        Get
            Return msg
        End Get
    End Property


    ' ---------------------------------------------------------- gets the messages's status value
    '
    Public ReadOnly Property Status() As Integer Implements IMidiMessage.Status
        Get
            Return UnpackStatus(msg)
        End Get
    End Property


    Public MustOverride ReadOnly Property MessageType() As MessageType Implements IMidiMessage.MessageType

End Class
