

Imports System.Collections
Imports System.Drawing
Imports System.Threading
Imports System.Windows.Forms

Namespace UI

    Partial Public Class PianoControl
        Inherits Control

        Private Enum KeyType
            White
            Black
        End Enum

        Private Shared ReadOnly keyTable As New Hashtable()

        Private Shared ReadOnly KeyTypeTable As KeyType() = {KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, _
         KeyType.White, KeyType.Black, KeyType.White, KeyType.Black, KeyType.White, KeyType.White, _
         KeyType.Black, KeyType.White}

        Private Delegate Sub NoteMessageCallback(ByVal message As ChannelMessage)

        Private Const DefaultLowNoteID As Integer = 21

        Private Const DefaultHighNoteID As Integer = 109

        Private Const BlackKeyScale As Double = 0.666666666

        Private context As SynchronizationContext

        Private m_lowNoteID As Integer = DefaultLowNoteID

        Private m_highNoteID As Integer = DefaultHighNoteID

        Private octaveOffset As Integer = 5

        Private m_noteOnColor As Color = Color.SkyBlue

        Private m_noteBlackOnColor As Color = Color.DarkBlue

        Private keys As PianoKey() = Nothing

        Private whiteKeyCount As Integer

        Private noteOnCallback As NoteMessageCallback

        Private noteOffCallback As NoteMessageCallback

        Public Event PianoKeyDown As EventHandler(Of PianoKeyEventArgs)

        Public Event PianoKeyUp As EventHandler(Of PianoKeyEventArgs)

        Shared Sub New()
            keyTable.Add(Windows.Forms.Keys.A, 0)
            keyTable.Add(Windows.Forms.Keys.W, 1)
            keyTable.Add(Windows.Forms.Keys.S, 2)
            keyTable.Add(Windows.Forms.Keys.E, 3)
            keyTable.Add(Windows.Forms.Keys.D, 4)
            keyTable.Add(Windows.Forms.Keys.F, 5)
            keyTable.Add(Windows.Forms.Keys.T, 6)
            keyTable.Add(Windows.Forms.Keys.G, 7)
            keyTable.Add(Windows.Forms.Keys.Y, 8)
            keyTable.Add(Windows.Forms.Keys.H, 9)
            keyTable.Add(Windows.Forms.Keys.U, 10)
            keyTable.Add(Windows.Forms.Keys.J, 11)
            keyTable.Add(Windows.Forms.Keys.K, 12)
            keyTable.Add(Windows.Forms.Keys.O, 13)
            keyTable.Add(Windows.Forms.Keys.L, 14)
            keyTable.Add(Windows.Forms.Keys.P, 15)
        End Sub

        Public Sub New()
            CreatePianoKeys()
            InitializePianoKeys()
            context = SynchronizationContext.Current
            noteOnCallback = AddressOf noteOnCallbackDelegate
            noteOffCallback = AddressOf noteOffCallbackDelegate
        End Sub

        Private Sub noteOnCallbackDelegate(ByVal message As ChannelMessage)
            If message.Data2 > 0 Then
                keys(message.Data1 - m_lowNoteID).PressPianoKey()
            Else
                keys(message.Data1 - m_lowNoteID).ReleasePianoKey()
            End If
        End Sub
        Private Sub noteOffCallbackDelegate(ByVal message As ChannelMessage)
            keys(message.Data1 - m_lowNoteID).ReleasePianoKey()
        End Sub

        Private Sub CreatePianoKeys()
            ' ------------------------------------------------------------ if already created
            If keys IsNot Nothing Then
                ' -------------------------------------------------------- remove and dispose of current piano keys
                For Each key As PianoKey In keys
                    Controls.Remove(key)
                    key.Dispose()
                Next
            End If

            keys = New PianoKey(HighNoteID - LowNoteID) {}

            whiteKeyCount = 0

            For i As Integer = 0 To keys.Length - 1
                keys(i) = New PianoKey(Me)
                keys(i).NoteID = i + LowNoteID

                If KeyTypeTable(keys(i).NoteID) = KeyType.White Then
                    whiteKeyCount += 1
                Else
                    keys(i).NoteOffColor = Color.Black
                    keys(i).BringToFront()
                End If

                keys(i).NoteOnColor = NoteOnColor

                Controls.Add(keys(i))
            Next
        End Sub

        Private Sub InitializePianoKeys()
            If keys.Length = 0 Then
                Return
            End If

            Dim whiteKeyWidth As Integer = Width \ whiteKeyCount
            Dim blackKeyWidth As Integer = CInt(whiteKeyWidth * BlackKeyScale)
            Dim blackKeyHeight As Integer = CInt(Height * BlackKeyScale)
            Dim offset As Integer = whiteKeyWidth - blackKeyWidth \ 2
            Dim n As Integer = 0
            Dim w As Integer = 0

            While n < keys.Length
                If KeyTypeTable(keys(n).NoteID) = KeyType.White Then
                    keys(n).IsBlackKey = False
                    keys(n).Height = Height
                    keys(n).Width = whiteKeyWidth
                    keys(n).Location = New Point(w * whiteKeyWidth, 0)
                    w += 1
                    n += 1
                Else
                    keys(n).IsBlackKey = True
                    keys(n).Height = blackKeyHeight
                    keys(n).Width = blackKeyWidth
                    keys(n).Location = New Point(offset + (w - 1) * whiteKeyWidth)
                    keys(n).BringToFront()
                    n += 1
                End If
            End While
        End Sub

        Public Sub Send(ByVal message As ChannelMessage)
            If message.Command = ChannelCommand.NoteOn AndAlso message.Data1 >= LowNoteID AndAlso message.Data1 <= HighNoteID Then
                If InvokeRequired Then
                    BeginInvoke(noteOnCallback, message)
                Else
                    noteOnCallback(message)
                End If
            ElseIf message.Command = ChannelCommand.NoteOff AndAlso message.Data1 >= LowNoteID AndAlso message.Data1 <= HighNoteID Then
                If InvokeRequired Then
                    BeginInvoke(noteOffCallback, message)
                Else
                    noteOffCallback(message)
                End If
            End If
        End Sub

        Public Sub PressPianoKey(ByVal noteID As Integer)
            If noteID >= m_lowNoteID AndAlso noteID <= m_highNoteID Then
                keys(noteID - m_lowNoteID).PressPianoKey()
            End If
        End Sub

        Public Sub ReleasePianoKey(ByVal noteID As Integer)
            If noteID >= m_lowNoteID AndAlso noteID <= m_highNoteID Then
                keys(noteID - m_lowNoteID).ReleasePianoKey()
            End If
        End Sub

        Public Sub PressPianoKey(ByVal k As Windows.Forms.Keys)
            If Not Focused Then
                Return
            End If

            If keyTable.Contains(k) Then
                Dim noteID As Integer = CInt(keyTable(k)) + 12 * octaveOffset

                If noteID >= LowNoteID AndAlso noteID <= HighNoteID Then
                    If Not keys(noteID - m_lowNoteID).IsPianoKeyPressed Then
                        keys(noteID - m_lowNoteID).PressPianoKey()
                    End If
                End If
            Else
                If k = Windows.Forms.Keys.D0 Then
                    octaveOffset = 0
                ElseIf k = Windows.Forms.Keys.D1 Then
                    octaveOffset = 1
                ElseIf k = Windows.Forms.Keys.D2 Then
                    octaveOffset = 2
                ElseIf k = Windows.Forms.Keys.D3 Then
                    octaveOffset = 3
                ElseIf k = Windows.Forms.Keys.D4 Then
                    octaveOffset = 4
                ElseIf k = Windows.Forms.Keys.D5 Then
                    octaveOffset = 5
                ElseIf k = Windows.Forms.Keys.D6 Then
                    octaveOffset = 6
                ElseIf k = Windows.Forms.Keys.D7 Then
                    octaveOffset = 7
                ElseIf k = Windows.Forms.Keys.D8 Then
                    octaveOffset = 8
                ElseIf k = Windows.Forms.Keys.D9 Then
                    octaveOffset = 9
                End If
            End If
        End Sub

        Public Sub ReleasePianoKey(ByVal k As Keys)
            If Not keyTable.Contains(k) Then
                Return
            End If

            Dim noteID As Integer = CInt(keyTable(k)) + 12 * octaveOffset

            If noteID >= LowNoteID AndAlso noteID <= HighNoteID Then
                keys(noteID - m_lowNoteID).ReleasePianoKey()
            End If
        End Sub

        Protected Overloads Overrides Sub OnResize(ByVal e As EventArgs)
            InitializePianoKeys()
            MyBase.OnResize(e)
        End Sub

        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                For Each key As PianoKey In keys
                    key.Dispose()
                Next
            End If
            MyBase.Dispose(disposing)
        End Sub

        Protected Overridable Sub OnPianoKeyDown(ByVal e As PianoKeyEventArgs)
            RaiseEvent PianoKeyDown(Me, e)
        End Sub

        Protected Overridable Sub OnPianoKeyUp(ByVal e As PianoKeyEventArgs)
            RaiseEvent PianoKeyUp(Me, e)
        End Sub

        Public Property LowNoteID() As Integer
            Get
                Return m_lowNoteID
            End Get
            Set(ByVal value As Integer)
                If value < 0 OrElse value > ShortMessage.DataMaxValue Then
                    Throw New ArgumentOutOfRangeException("LowNoteID", value, "Low note ID out of range.")
                End If
                If value = m_lowNoteID Then
                    Return
                End If

                m_lowNoteID = value

                If m_lowNoteID > m_highNoteID Then
                    m_highNoteID = m_lowNoteID
                End If

                CreatePianoKeys()
                InitializePianoKeys()
            End Set
        End Property

        Public Property HighNoteID() As Integer
            Get
                Return m_highNoteID
            End Get
            Set(ByVal value As Integer)
                If value < 0 OrElse value > ShortMessage.DataMaxValue Then
                    Throw New ArgumentOutOfRangeException("HighNoteID", value, "High note ID out of range.")
                End If

                If value = m_highNoteID Then
                    Return
                End If

                m_highNoteID = value

                If m_highNoteID < m_lowNoteID Then
                    m_lowNoteID = m_highNoteID
                End If

                CreatePianoKeys()
                InitializePianoKeys()
            End Set
        End Property

        Public Property NoteOnColor() As Color
            Get
                Return m_noteOnColor
            End Get
            Set(ByVal value As Color)
                If value = m_noteOnColor Then
                    Return
                End If

                m_noteOnColor = value

                For Each key As PianoKey In keys
                    key.NoteOnColor = m_noteOnColor
                Next
            End Set
        End Property

        Public Property NoteBlackOnColor() As Color
            Get
                Return m_noteBlackOnColor
            End Get
            Set(ByVal value As Color)
                If value = m_noteBlackOnColor Then
                    Return
                End If

                m_noteBlackOnColor = value

                For Each key As PianoKey In keys
                    key.NoteBlackOnColor = m_noteBlackOnColor
                Next
            End Set
        End Property

    End Class

End Namespace
